
#####################
##### Libraries #####
#####################

library(readr)
library(vroom)
library(janitor)
library(tidyverse)
library(lubridate)
library(gghalves)
library(ggdist)
library(patchwork)
library(summarytools)
library(kableExtra)
library(cccrm)
library(MethodCompare)
library(Deriv)
library(rmcorr)

# Load only if calibrating or processing raw accelerometer data
#library(GENEAread)
#library(GENEAclassify)
#library(GGIR)



########################################################
##### Calibration of raw acceleration measurements #####
########################################################

# title: "GENEActiv calibration 09-12-21"
# authors: "Charles Sweetland, Joss Langford"
# copyright: "Activinsights Ltd, 2021"

# Clear the decks
rm(list = ls())
gc()

Sys.Date()

# store initial working directory
Basedir = getwd()

# Set Data folder
Data = paste0(Basedir,"/Data/")

# Set Uncal folder
Uncal = paste0(Basedir,"/Uncal/")

# Finding the bin files inside the Input folder
BinPattern <- "*\\.[bB][iI][nN]$"
files = list.files(path = Uncal, pattern = BinPattern, full.names = TRUE, recursive = TRUE)
NoFiles = length(files)
if (NoFiles == 0){stop("There are no files in the data directory")}

# Initialise parameters
Scale_X = Scale_Y = Scale_Z = Offset_X = Offset_Y = Offset_Z = TempOffset_X = TempOffset_Y = TempOffset_Z = QCMessage = c()

#Binfile[j] = files[j]
filenames = gsub(".bin", "", basename(files))

for (j in 1:NoFiles){ 
  Binfiles = files[j]
  
  # Running the calibration function 
  C = GENEActiv.calibrate(Binfiles, 
                          use.temp = TRUE,
                          spherecrit = 0.3, 
                          minloadcrit = 72,
                          printsummary = FALSE, 
                          chunksize=c(0.5), 
                          windowsizes=c(60,900,3600))
  
  if (C$QCmessage != "recalibration not done because not enough points on all sides of the sphere"){
    #### Then adding this to the overall csv file writing this out to a csv file #### 
    digits = 6
    Scale_X[j] = round(C$scale[1], digits = digits)
    Scale_Y[j] = round(C$scale[2], digits = digits)
    Scale_Z[j] = round(C$scale[3], digits = digits)
    # Offset
    Offset_X[j] = round(C$offset[1], digits = digits)
    Offset_Y[j] = round(C$offset[2], digits = digits)
    Offset_Z[j] = round(C$offset[3], digits = digits)
    # Temp offset
    TempOffset_X[j] = round(C$tempoffset[1], digits = digits)
    TempOffset_Y[j] = round(C$tempoffset[2], digits = digits)
    TempOffset_Z[j] = round(C$tempoffset[3], digits = digits)
    
    # Message 
    QCMessage[j] = as.character(C$QCmessage[1])
    
    #### Creating a single csv for each file ####
    digits = 6
    df = data.frame(digits = 6,
                    file = Binfiles,
                    Scale_X = Scale_X[j],
                    Scale_Y = Scale_Y[j],
                    Scale_Z = Scale_Z[j],
                    # Offset
                    Offset_X = Offset_X[j],
                    Offset_Y = Offset_Y[j],
                    Offset_Z = Offset_Z[j],
                    # Temp offset
                    TempOffset_X = TempOffset_X[j],
                    TempOffset_Y = TempOffset_Y[j],
                    TempOffset_Z = TempOffset_Z[j],
                    
                    QCMessage = QCMessage[j])
  } else {
    
    digits = 6      
    Scale_X[j] = round(C$scale[1], digits = digits)
    Scale_Y[j] = round(C$scale[2], digits = digits)
    Scale_Z[j] = round(C$scale[3], digits = digits)
    # Offset
    Offset_X[j] = round(C$offset[1], digits = digits)
    Offset_Y[j] = round(C$offset[2], digits = digits)
    Offset_Z[j] = round(C$offset[3], digits = digits)
    # Temp offset
    TempOffset_X[j] = NA
    TempOffset_Y[j] = NA
    TempOffset_Z[j] = NA
    
    # Message 
    QCMessage[j] = as.character(C$QCmessage[1])
    
    #### Creating a single csv for each file ####
    digits = 6
    df = data.frame(digits = 6,
                    file = Binfiles,
                    Scale_X = Scale_X[j],
                    Scale_Y = Scale_Y[j],
                    Scale_Z = Scale_Z[j],
                    # Offset
                    Offset_X = Offset_X[j],
                    Offset_Y = Offset_Y[j],
                    Offset_Z = Offset_Z[j],
                    # Temp offset
                    TempOffset_X = TempOffset_X[j],
                    TempOffset_Y = TempOffset_Y[j],
                    TempOffset_Z = TempOffset_Z[j],
                    
                    QCMessage = QCMessage[j])
  }
  
  
  # Write this CSV out if needed
  #write.csv(df, paste0(Data,filenames[j],"_Calibration_Values.csv"))
  
  # Read in the current values - 
  Lines=readLines(Binfiles,-1) # Reads the bin file to the point where the calibration data is.
  XOffset = Lines[49];  XGain = Lines[48]
  YOffset = Lines[51];  YGain = Lines[50]
  ZOffset = Lines[53];  ZGain = Lines[52]
  
  # Extracting just the numerical value. - Offset
  XOffsetN = as.numeric(unlist(strsplit(XOffset,"x offset:"))[2])
  YOffsetN = as.numeric(unlist(strsplit(YOffset,"y offset:"))[2])
  ZOffsetN = as.numeric(unlist(strsplit(ZOffset,"z offset:"))[2])
  
  # Extracting just the numerical value for the Gain
  XGainN = as.numeric(unlist(strsplit(XGain,"x gain:"))[2])
  YGainN = as.numeric(unlist(strsplit(YGain,"y gain:"))[2])
  ZGainN = as.numeric(unlist(strsplit(ZGain,"z gain:"))[2])
  
  # Calculating the New values using the calibration values- Rounding to whole numbers
  if (C$offset[1] != 0){
    #        XOffsetNew = round((XOffsetN * C$offset[1]) * 25.6)
    XOffsetNew = round(XOffsetN - C$offset[1] * 256 * 100)
    Lines[49] = paste("x offset:", sep = "", XOffsetNew)}
  
  if (C$offset[2] != 0){
    #        YOffsetNew = round((YOffsetN * C$offset[2]) * 25.6)
    YOffsetNew = round(YOffsetN -  C$offset[2] * 256 * 100)
    Lines[51] = paste("y offset:", sep = "", YOffsetNew)
  }
  
  if (C$offset[2] != 0){
    #        ZOffsetNew = round((ZOffsetN * C$offset[3]) * 25.6)
    ZOffsetNew = round(ZOffsetN - C$offset[3] * 256 * 100)
    Lines[53] = paste("z offset:", sep = "", ZOffsetNew)
  }
  
  if (C$scale[1] != 0){
    XGainNew = round(XGainN / C$scale[1])
    Lines[48] = paste("x gain:", sep = "", XGainNew)
  }
  
  if (C$scale[2] != 0){
    YGainNew = round(YGainN / C$scale[2])
    Lines[50] = paste("y gain:", sep = "", YGainNew)
  }
  if (C$scale[3] != 0){
    ZGainNew = round(ZGainN / C$scale[3])
    Lines[52] = paste("z gain:", sep = "", ZGainNew)
  }
  
  # Create the calibrated file name
  con <- file(paste0(Data, filenames[j], "_Calibrated.bin"))
  write(Lines, con)
  close (con)
} 

Stats = data.frame(filenames,
                   Scale_X, 
                   Scale_Y, 
                   Scale_Z, 
                   Offset_X, 
                   Offset_Y, 
                   Offset_Z, 
                   TempOffset_X, 
                   TempOffset_Y, 
                   TempOffset_Z, 
                   QCMessage)

colnames(Stats) = c("File",
                    "Scale_X",
                    "Scale_Y", 
                    "Scale_Z", 
                    "Offset_X", 
                    "Offset_Y", 
                    "Offset_Z", 
                    "TempOffset_X", 
                    "TempOffset_Y", 
                    "TempOffset_Z", 
                    "Message")

# Writing out the CSV 
write.csv(Stats, 
          paste0(Data, "Calibration_Values.csv"), 
          row.names = F)



#########################################
##### GENEActiv raw data processing #####
#########################################

# A copy of the latest analysis tool for batch-processing of raw accelerometer data
# with the open-source GENEAcount algorithm can be obtained from:
# https://activinsights.com/support/geneactiv-support/



########################################
##### GGIR processing (valid days) #####
########################################

# Process binary data with GGIR parts 1 and 2 to identify valid hours in each day
GGIR(datadir = 'LINK TO CALIBRATED GENEACTIV RAW DATA', # Path to folder with calibrated raw data
     outputdir = 'LINK TO OUTPUT FOLDER',
     mode = 1:2, # Run GGIR parts 1 and 2
     do.report = 2, # Generate csv-report for GGIR part 2
     qwindow = c(0, 24)) # Analyse midnight to midnight (full day segments only)




#####################################
###### Prepare valid day input ######
#####################################

### ActivPAL

# Import valid day and daily summaries reports
# Valid Day Report
pal_validdays <- read_csv('LINK TO "VALID DAY" REPORT FROM PALBATCH') %>%
  clean_names()

# Daily Summaries Report
pal_dailysumm <- read_csv('LINK TO "DAILY SUMMARIES" REPORT FROM PALBATCH') %>%
  clean_names()

# Pull valid/invalid days from daily summary report
activpal_valid_days <- pal_dailysumm %>%
  select(c(filename, date, valid_day)) %>%
  # Turn file name column into participant ID based on file name
  mutate(filename = str_extract(filename, '[P]{1}[0-9]{3}')) %>%
  rename(participant_id = filename) %>%
  # Update date format
  mutate(date = as.Date(dmy(date)))



### GENEActiv

# Create valid/invalid day identifier for each day across all participants
# Read files and add an ID column with participant IDs
geneactiv_valid_days <- vroom(file = 'LINK TO "PART2 DAYSUMMARY" FROM GGIR', 
                              col_types = cols(calendar_date = col_date(format = '%Y-%m-%dT%Z')),
                              .name_repair = ~ janitor::make_clean_names(., case = 'snake')) %>%
  
  # Clean and calculate additional measures
  # Turn filename column into participant ID based on file name
  mutate(filename = str_extract(filename, '[P]{1}[0-9]{3}')) %>%
  rename(participant_id = filename) %>%
  # Make calendar_date a date-only column (remove time)
  mutate(date = as.Date(calendar_date), 
         .after = calendar_date) %>%
  # GGIR mis-labels the last day for each participant because of a confirmed bug in v2.7-1 
  # The last day is incorrectly labelled as a duplicate of the previous day
  # To correct this, add a day to each day where this happens
  group_by(participant_id) %>%
  mutate('date' = case_when(duplicated(date) ~ date + 1,
                            TRUE ~ date)) %>%
  ungroup() %>%
  # Remove unnecessary columns
  select(-c(1, 3, 5, 9:21)) %>%
  # Add column with valid/invalid day indicator matching activPAL 24hr protocol (max. 4hrs non-wear)
  mutate(valid_day = ifelse(n_valid_hours <= 20, 0, 1))



### Combine valid day information for both systems
valid_days <- geneactiv_valid_days %>%
  # Select relevant columns
  select(c('participant_id', 'date', 'weekday', 'valid_day')) %>%
  # Combine both data frames
  left_join(activpal_valid_days,
            by = c('participant_id', 'date')) %>%
  # Fix column names
  rename(geneactiv_valid_day = valid_day.x,
         activpal_valid_day = valid_day.y) %>%
  # Make NAs introduced by different column lengths 0 = invalid day
  replace_na(list(activpal_valid_day = 0)) %>%
  # Add column which indicates that the day is valid for BOTH, GENEActiv and activPAL
  # (0 = invalid, 1 = valid)
  mutate(both_valid_day = case_when(geneactiv_valid_day == 1 & 
                                      activpal_valid_day == 1 ~ 1,
                                    TRUE ~ 0))



##############################################
##### GENEActiv data import and cleaning #####
##############################################

# Import GENEActiv data for all participants

# Prepare list of file paths
gen_files <- fs::dir_ls(path = 'LINK TO FOLDER CONTAINING "ALL DATA" CSVs FROM MARKDOWN OUTPUT')

# Read files and add an ID column with participant IDs
geneactiv_data <- vroom(gen_files, 
                        delim = ',',
                        id = 'path',
                        .name_repair = ~ janitor::make_clean_names(., case = 'snake'))

# Clean data and add additional columns
geneactiv_data <- geneactiv_data %>%
  
  # Turn path column into participant ID based on file name
  mutate(path = str_extract(path, '[P]{1}[0-9]{3}')) %>%
  rename(participant_id = path) %>%
  
  # Delete unnecessary columns
  select(-c(2:6, 8:20, 22:24)) %>%
  
  # Rename columns for consistency across GENEActiv and activPAL
  rename(duration_s = segment_duration,
         num_steps = step_gene_acount) %>%
  
  # Filter out events without steps 
  # (equivalent to removing event type 0 and 1 from the activPAL reports)
  filter(num_steps >= 1) %>%
  
  # Add device column
  mutate(device = as.factor(rep('Wrist')),
         .after = participant_id) %>%
  
  # Add column with invalid day indicator
  left_join(select(geneactiv_valid_days, 
                   c('participant_id', 'date', 'valid_day')),
            by = c('participant_id', 'date')) %>%
  mutate(valid_day = as.factor(valid_day)) %>%
  
  # Move device, date and valid_day columns
  relocate(date, .after = device) %>%
  relocate(valid_day, .after = date) %>%
  
  # Add measurement day column
  group_by(participant_id) %>%
  mutate(measurement_day = cumsum(c(1, as.numeric(diff(date)))),
         .after = device) %>%
  ungroup() %>%
  
  # Calculate cadence for each event
  mutate(cadence =  num_steps / duration_s * 60) %>%
  
  # Filter out invalid cadences 
  filter(cadence >= 20, # < 20 steps/min not valid estimates
         cadence <= 175) %>% # > 175 steps/min represent workouts, not walking
  # Round to full steps
  mutate(cadence = round(cadence))



#############################################
##### ActivPAL data import and cleaning #####
#############################################

# Read all .csv files into one data frame and clean column names
# Prepare list of file paths
pal_files <- fs::dir_ls(path = 'LINK TO FOLDER CONTAINING "STEPPING BOUTS" CSVs FROM PALBATCH')

# Read files and add an ID column with participant IDs
# Files need to be CSV UTF-8 (Comma-delimited) 
# format and not default PAL .csv output
activpal_data <- vroom(pal_files, 
                       delim = ',', 
                       id = 'path',
                       .name_repair = ~ janitor::make_clean_names(., case = 'snake'))

# Clean data and add additional columns
activpal_data <- activpal_data %>%
  
  # Turn path column into participant ID based on file name
  mutate(path = str_extract(path, '[P]{1}[0-9]{3}')) %>%
  rename(participant_id = path) %>%
  
  # Filter out sedentary and upright events without steps
  filter(event_type == 2) %>%
  
  # Add date-only column (without time component)
  mutate(date = as.Date(dmy_hm(time_approx)), 
         .after = participant_id) %>%
  
  # Remove unnecessary columns
  select(c('participant_id', 'date', 'duration_s', 'num_steps', 'cadence')) %>%
  
  # Add device column
  mutate(device = as.factor(rep('Thigh')),
         .after = participant_id) %>%
  
  # Add column with invalid day indicator
  left_join(activpal_valid_days,
            by = c('participant_id', 'date')) %>%
  mutate(valid_day = as.factor(valid_day)) %>%
  relocate(valid_day, .after = date) %>%
  
  # Add measurement day column
  group_by(participant_id) %>%
  mutate(measurement_day = cumsum(c(1, as.numeric(diff(date)))),
         .after = valid_day) %>%
  ungroup()



##########################################
##### Merging GENEActiv and activPAL #####
##########################################

# Merge GENEActiv and activPAL events-level data in a new data frame
combined_events <- bind_rows(geneactiv_data, activpal_data)

# Handle valid/invalid days across both devices to make data sets comparable only days 
# which are valid for BOTH devices can be included
combined_events <- combined_events %>%
  # Replace separate valid day indicator with joint indicator
  left_join(select(valid_days, 'participant_id', 'date', 'both_valid_day'),
            by = c('participant_id', 'date')) %>%
  # Remove old valid_day column and relocate the new one
  select(-'valid_day') %>%
  relocate('both_valid_day', .after = 'date') %>%
  # Filter out invalid days (0 = invalid, 1 = valid)
  filter(both_valid_day == 1) %>%
  
  # Remove all events with less than 10 consecutive steps
  filter(num_steps >= 10) %>%
  
  # Set device column factor order
  mutate(device = factor(device, levels = c('Thigh', 'Wrist'))) %>%
  
  # Update measurement_day column
  group_by(participant_id) %>%
  mutate(measurement_day = cumsum(c(1, as.numeric(diff(date)))),
         .after = both_valid_day) %>%
  ungroup() %>%
  
  # Remove all days beyond the seventh day
  filter(measurement_day < 8) %>%
  
  ### Calculate ggregate measures

  # Add column with total number of steps and total duration for each participant
  group_by(participant_id, device) %>%
  # Total steps
  mutate(total_steps = sum(num_steps),
         .after = num_steps) %>%
  # Total duration in minutes
  mutate(total_duration_m = sum(duration_s / 60),
         .after = duration_s) %>%
  ungroup() %>%
  
  # Calculate total stepping time per device per participant per day
  group_by(participant_id, device, date) %>%
  mutate(daily_stepping_time_m = sum(duration_s) / 60,
         .after = duration_s) %>%
  ungroup() %>%
  
  # Calculate total and mean steps per device per day for each participant
  # Total steps on each day
  group_by(participant_id, device, date) %>%
  mutate(total_steps_on_day = sum(num_steps), 
         .after = total_steps) %>%
  ungroup() %>%
  
  # Calculate total steps accumulated in events of different lengths
  group_by(participant_id, device, date) %>%
  mutate('steps_<=1_min' = sum(num_steps[duration_s <= 60])) %>%
  mutate('steps_>1_<=5_min' = sum(num_steps[duration_s > 60 & duration_s <= 5*60])) %>%
  mutate('steps_>5_<=10_min' = sum(num_steps[duration_s > 5*60 & duration_s <= 10*60])) %>%
  mutate('steps_>10_<=20_min' = sum(num_steps[duration_s > 10*60 & duration_s <= 20*60])) %>%
  mutate('steps_>20_<=30_min' = sum(num_steps[duration_s > 20*60 & duration_s <= 30*60])) %>%
  mutate('steps_>30_min' = sum(num_steps[duration_s > 30*60])) %>%
  ungroup() %>%
  
  # Calculate steps accumulated in upper and lower half of cadences per participant per day
  # Calculate median by device, across all participants and days
  group_by(device) %>%
  # Calculate median per device for cadences equal to or above 45 steps/min
  mutate(median_cadence = median(cadence[cadence >= 45])) %>%
  ungroup() %>%
  # For each day and participant, calculate device-specific steps below and above 
  # the device-specific medians as well as steps below 45 steps per minute
  group_by(participant_id, device, date) %>%
  mutate(cadence_bottom_steps = sum(num_steps[cadence < 45]),
         cadence_lower_steps = sum(num_steps[cadence >= 45 & cadence <= median_cadence]),
         cadence_upper_steps = sum(num_steps[cadence >= 45 & cadence > median_cadence])) %>%
  ungroup() %>%
  
  # Label each event based on whether it's in the upper half or lower half, 
  # on median or below 45 steps per minute by device
  group_by(device) %>%
  mutate(cadence_median_split = 
           case_when(cadence <= median(cadence[cadence >= 45]) ~ 'Lower half',
                     cadence > median(cadence[cadence >= 45]) ~ 'Upper half'),
         .after = cadence) %>%
  # Define factor levels
  mutate(cadence_median_split = factor(cadence_median_split, 
                                       levels = c('Lower half',
                                                  'Upper half'))) %>%
  ungroup() %>%
  
  # Remove invalid day column now that it's no longer needed
  select(-both_valid_day)



##############################
##### Summary statistics #####
##############################

# Number of participants after cleaning
combined_events %>%
  distinct(participant_id) %>%
  tally()

# Summary statistics for daily steps by device
combined_events %>%
  distinct(participant_id, date, device, total_steps_on_day) %>%
  select(device, total_steps_on_day) %>%
  group_by(device) %>%
  descr(stats = c('mean', 'sd', 'min', 'q1', 'med', 'q3', 'max'),
        order = 'preserve',
        transpose = TRUE,
        justify = 'r',
        style = 'rmarkdown')

# Summary statistics table by device for event-level metrics
summary_table <- combined_events %>%
  select(c('device', 'num_steps', 'duration_s', 'cadence')) %>%
  rename('Cadence (steps/min)' = cadence,
         'Duration (seconds)' = duration_s,
         'Step count' = num_steps) %>%
  group_by(device) %>%
  descr(stats = c('mean', 'sd', 'min', 'q1', 'med', 'q3', 'max'),
        order = 'preserve',
        transpose = TRUE,
        justify = 'r',
        style = 'rmarkdown')

sum_table_thigh <- t(round(summary_table$`device = Thigh`, 2))
sum_table_wrist <- t(round(summary_table$`device = Wrist`, 2))

sum_table <- cbind(sum_table_thigh, sum_table_wrist)

kable(sum_table, 
      caption = 'Title',
      booktabs = TRUE) %>%
  column_spec(1, width = '2cm', bold = TRUE) %>%
  column_spec(2, width = '2cm') %>%
  column_spec(3, width = '2cm') %>%
  column_spec(4, width = '2cm') %>%
  column_spec(5, width = '2.5cm') %>%
  column_spec(6, width = '2cm') %>%
  column_spec(7, width = '2cm') %>%
  add_header_above(c('',
                     'Thigh' = 3,
                     'Wrist' = 3)) %>%
  kable_styling(latex_options = c('striped', 'hold_position'),
                full_width = FALSE)

# Stepping events per day per person
combined_events %>%
  select(participant_id, device, date) %>%
  group_by(participant_id, device, date) %>%
  tally() %>%
  ungroup() %>%
  group_by(device) %>%
  select(device, n) %>% 
  descr(stats = c('mean', 'sd', 'min', 'q1', 'med', 'q3', 'max'),
        order = 'preserve',
        transpose = TRUE,
        justify = 'r',
        style = 'rmarkdown')

# Stepping time per day per person
combined_events %>%
  select(participant_id, device, date, daily_stepping_time_m) %>%
  distinct(participant_id, device, date, daily_stepping_time_m) %>%
  select(device, daily_stepping_time_m) %>%
  group_by(device) %>%
  descr(stats = c('mean', 'sd', 'min', 'q1', 'med', 'q3', 'max'),
        order = 'preserve',
        transpose = TRUE,
        justify = 'r',
        style = 'rmarkdown')

# Summary statistics table by device for slower and faster walking
combined_events %>%
  select(c('device', 'cadence_lower_steps', 'cadence_upper_steps')) %>%
  rename('Slower walking' = cadence_lower_steps,
         'Faster walking' = cadence_upper_steps) %>%
  group_by(device) %>%
  descr(stats = c('mean', 'sd', 'min', 'q1', 'med', 'q3', 'max'),
        order = 'preserve',
        transpose = TRUE,
        justify = 'r',
        style = 'rmarkdown')


######################################
##### Explorative visualisations #####
######################################

### Raincloud plot of total steps per day across all participants
combined_events %>%
  distinct(participant_id, 
           across(c('device', 
                    'total_steps_on_day'))) %>%
  # Plot
  ggplot(., aes(x = device, 
                y = total_steps_on_day, 
                fill = device)) + 
  stat_halfeye(adjust = 0.4, 
               width = 0.5, 
               .width = 0, 
               justification = -0.3, 
               point_colour = NA) + 
  geom_boxplot(width = 0.2,
               outlier.shape = NA) +
  geom_half_point(aes(colour = device),
                  side = "l",
                  position = position_nudge(x = -0.02),
                  range_scale = 0.3, 
                  alpha = 0.4,
                  size = 1) + 
  scale_x_discrete(limits = rev) +
  scale_y_continuous(labels = scales::comma) +
  labs(title = 'Total daily steps by device',
       subtitle = 'Each point represents one day',
       x = '', 
       y = 'Steps per day') +
  coord_flip() +
  theme_bw() +
  theme(legend.position = 'none')



### Event-level histograms for step count, duration and cadence

# Steps per event
hist_events <- combined_events %>%
  ggplot(., aes(x = num_steps, 
                fill = device)) +
  geom_histogram(breaks = c(seq(0, 200, by = 5), max(combined_events$num_steps)),
                 alpha = 0.6,
                 position = 'identity') +
  coord_cartesian(xlim = c(0, 205)) +
  scale_x_continuous(breaks = scales::pretty_breaks(n = 6)) +
  scale_y_continuous(labels = scales::comma,
                     breaks = scales::pretty_breaks(n = 6)) +
  labs(title = '(A) Steps',
       subtitle = 'Overflow bin for events with > 200 steps',
       x = 'Steps per event',
       y = 'Count') +
  guides(fill = guide_legend(title = '')) +
  theme_bw() +
  theme(legend.position = 'bottom')

# Duration of each event IN SECONDS
hist_duration_s <- combined_events %>%
  ggplot(., aes(x = duration_s, 
                fill = device)) +
  geom_histogram(breaks = c(seq(0, 240, by = 5), max(combined_events$duration_s)),
                 alpha = 0.6,
                 position = 'identity') +
  coord_cartesian(xlim = c(0, 245)) +
  scale_x_continuous(breaks = scales::pretty_breaks(n = 10)) +
  scale_y_continuous(labels = scales::comma,
                     breaks = scales::pretty_breaks(n = 6)) +
  labs(title = '(B) Duration',
       subtitle = 'Overflow bin for events longer than 240 secs',
       x = 'Duration of event (seconds)',
       y = '') +
  guides(fill = guide_legend(title = '')) +
  theme_bw() +
  theme(legend.position = 'bottom')

# Duration of each event IN MINUTES
hist_duration_m <- combined_events %>%
  ggplot(., aes(x = duration_s / 60, 
                fill = device)) +
  geom_histogram(breaks = c(seq(0, 10, by = 1), max(combined_events$duration_s / 60)),
                 alpha = 0.6,
                 position = 'identity') +
  coord_cartesian(xlim = c(0, 11)) +
  scale_x_continuous(breaks = scales::pretty_breaks(n = 6)) +
  scale_y_continuous(labels = scales::comma,
                     breaks = scales::pretty_breaks(n = 6)) +
  labs(title = 'event duration',
       subtitle = 'Overflow events >10 minutes',
       x = 'Duration of event (minutes)',
       y = 'Count') +
  guides(fill = guide_legend(title = '')) +
  theme_bw() +
  theme(legend.position = 'bottom')

# Cadences of each event
hist_cadence <- combined_events %>%
  ggplot(., aes(x = cadence, 
                fill = device)) +
  geom_histogram(breaks = c(seq(0, 180, by = 5), max(combined_events$cadence)),
                 alpha = 0.6,
                 position = 'identity') +
  scale_x_continuous(breaks = scales::pretty_breaks(n = 6)) +
  scale_y_continuous(labels = scales::comma,
                     breaks = scales::pretty_breaks(n = 8 )) +
  labs(title = '(C) Cadence',
       x = 'Cadence of event (steps/min)',
       y = 'Count') +
  guides(fill = guide_legend(title = '')) +
  theme_bw() +
  theme(legend.position = 'bottom')

# Combine  histograms
hist_events + hist_duration_s + hist_cadence + guide_area() +
  plot_layout(ncol = 2, 
              guides = 'collect') &
  theme(legend.position = 'bottom',
        legend.direction = 'vertical')



### Plot each participant's activity profile
combined_events %>% 
  distinct(across(c('participant_id', 'device', 'date', 
                    'measurement_day', 'total_steps_on_day'))) %>%
  # Plot
  ggplot(., aes(x = measurement_day, 
                y = total_steps_on_day, 
                fill = device)) +
  geom_bar(stat = 'identity', 
           position = 'dodge', 
           width = 0.6) +
  scale_y_continuous(labels = scales::comma, 
                     breaks = scales::pretty_breaks(n = 3)) +
  scale_x_continuous(breaks = scales::pretty_breaks(n = 10)) +
  labs(title = '',
       x = 'Measurement day', 
       y = 'Steps') +
  guides(fill = guide_legend(title = '')) +
  facet_wrap(~ participant_id,
             ncol = 7) +
  theme_bw() +
  theme(legend.position = 'bottom',
        panel.grid.minor.x = element_blank(),
        panel.grid.major.x = element_blank())



### Plot daily step counts for each of the three cadence categories
combined_events %>%
  distinct(participant_id, device, measurement_day,
           .keep_all = TRUE) %>%
  select(c(device, cadence_bottom_steps, cadence_lower_steps, cadence_upper_steps)) %>%
  group_by(device) %>%
  mutate('Non-walking' = mean(cadence_bottom_steps),
         'Slower-paced walking' = mean(cadence_lower_steps),
         'Faster-paced walking' = mean(cadence_upper_steps)) %>%
  ungroup() %>%
  select(c(device, 'Non-walking', 'Slower-paced walking', 'Faster-paced walking')) %>%
  distinct(device,
           .keep_all = TRUE) %>%
  pivot_longer(!device,
               names_to = 'cadence',
               values_to = 'steps') %>%
  mutate(cadence = factor(cadence,
                          levels = c('Faster-paced walking',
                                     'Slower-paced walking',
                                     'Non-walking'))) %>%
  ggplot(aes(x = device,
             y = steps,
             fill = cadence)) +
  geom_bar(stat = 'identity',
           position = 'stack',
           width = 0.4) +
  geom_text(aes(label = format(round(steps), big.mark = ',', scientific = FALSE)),
            position = position_stack(vjust = 0.5),
            size = 3.1) +
  annotate(geom = 'text', 
           x = 1, 
           y = 10000,
           size = 3.1,
           fontface = 2,
           label = 'Total: 9,065') +
  annotate(geom = 'text', 
           x = 2, 
           y = 10000,
           size = 3.1,
           fontface = 2,
           label = 'Total: 9,721') +
  scale_y_continuous(labels = scales::comma,
                     breaks = scales::pretty_breaks(n = 5)) +
  scale_fill_discrete_qualitative(palette = 'Dark 3',
                                  rev = TRUE) +
  labs(x = '',
       y = 'Mean steps/day',
       fill = 'Cadence category') +
  theme_bw()



###############################################
##### Concordance Correlation Coefficient #####
###############################################

#### CCC for longitudinal repeated measurements

# Update the ccclon() function from package cccrm to store the full model output to 
# enable residual checking. By default, the function from the package only saves the 
# summary, which doesn't contain all residuals and fitted values.
ccclon_cust <- function (dataset, ry, rind, rtime, rmet, covar = NULL, rho = 0, 
                         cl = 0.95, control.lme = list()) 
{
  dades <- data.frame(dataset)
  dades <- dades %>% dplyr::rename(y = all_of(ry), ind = all_of(rind), 
                                   met = all_of(rmet), time = all_of(rtime))
  dades$ind <- as.factor(dades$ind)
  dades$met <- as.factor(dades$met)
  dades$y <- as.numeric(dades$y)
  dades$time2 <- as.numeric(dades$time)
  dades$time <- as.factor(dades$time)
  form = y ~ met + time + met * time
  if (length(covar) > 0) {
    form <- as.formula(paste("y~met+time+met*time", paste(covar, 
                                                          sep = "+"), sep = "+"))
  }
  if ((rho != 0) & (rho != 1)) {
    stop("Rho must be 0(compound simmetry) or 1 (AR1)")
  }
  if (rho == 0) {
    model.lme <- lme(form, dades, random = list(ind = pdBlocked(list(~1, 
                                                                     pdIdent(form = ~-1 + met), pdIdent(form = ~-1 + time)))), 
                     control = control.lme, na.action = na.omit)
    if (is.character(model.lme$apVar) == TRUE) {
      stop("Non-positive definite approximate variance-covariance")
    }
    model <- summary(model.lme)
    vars <- attr(model$apVar, "Pars")
    SA <- s_exp(vars[1])
    SAB <- s_exp(vars[2])
    SAG <- s_exp(vars[3])
    SE <- s_exp(vars[4])
    model_full <- model.lme # THIS WAS ADDED TO RETAIN THE FULL MODEL OUTPUT
  }
  if (rho == 1) {
    model.lme <- lme(form, dades, random = list(ind = pdBlocked(list(~1, 
                                                                     pdIdent(form = ~-1 + met), pdIdent(form = ~-1 + time)))), 
                     correlation = corAR1(form = ~time2 | ind/met), control = control.lme, 
                     na.action = na.omit)
    if (is.character(model.lme$apVar) == TRUE) {
      stop("Non-positive definite approximate variance-covariance")
    }
    model <- summary(model.lme)
    vars <- attr(model$apVar, "Pars")
    SA <- s_exp(vars[1])
    SAB <- s_exp(vars[2])
    SAG <- s_exp(vars[3])
    SE <- s_exp(vars[5])
    model_full <- model.lme # THIS WAS ADDED TO RETAIN THE FULL MODEL OUTPUT
  }
  ns <- length(unique(dades$ind))
  nt <- length(unique(dades$time))
  nm <- length(unique(dades$met))
  if ((rho == 0) | (rho == 1)) {
    b <- as.matrix(model.lme$coef$fixed)
    nd <- nm * (nm - 1)/2
    C <- array(0, dim = c(length(b), nt * nm))
    k <- 0
    for (i in 1:nm) {
      for (j in 1:nt) {
        k <- k + 1
        C[, k] <- c(1, contrasts(dades$met)[i, ], contrasts(dades$time)[j, 
        ], c(contrasts(dades$met)[i, ] %*% t(contrasts(dades$time)[j, 
        ])))
      }
    }
    L <- array(0, dim = c(length(b), nt * nd))
    k <- 0
    for (i in 1:(nt * (nm - 1))) {
      for (j in (1:(nm - 1))) {
        if ((i + nt * j) <= (nt * nm)) {
          k <- k + 1
          L[, k] = C[, i] - C[, i + nt * j]
        }
      }
    }
    Sb <- model.lme$varFix
    difmed <- t(L) %*% b
    A <- L %*% t(L)
    aux1 <- (t(difmed) %*% difmed) - sum(diag((A %*% Sb)))
    SB <- max(aux1/(nm * (nm - 1) * nt), 0)
  }
  ccc <- icc3(SA, SAB, SAG, SE, SB)
  names(ccc) <- c("CCC")
  var.SB <- ((2 * sum(diag(((A %*% Sb)^2)))) + (4 * (t(b) %*% 
                                                       A %*% Sb %*% A %*% b)))/((nm * (nm - 1) * nt)^2)
  if (rho == 0) {
    STAU <- model$apVar
    D_tau <- matrix(c(d_exp(vars[1]), d_exp(vars[2]), d_exp(vars[3]), 
                      d_exp(vars[4])), ncol = 1)
  }
  if (rho == 1) {
    STAU <- model$apVar[c(1:3, 5), c(1:3, 5)]
    D_tau <- matrix(c(d_exp(vars[1]), d_exp(vars[2]), d_exp(vars[3]), 
                      d_exp(vars[5])), ncol = 1)
  }
  S <- array(NA, c(5, 5))
  S[1:4, 1:4] <- D_tau %*% t(D_tau) * STAU
  S[5, 5] <- var.SB
  S[1, 4] <- S[4, 1] <- (-1/ns) * (-1/ns) * (S[1, 2] + S[1, 3])
  S[2, 4] <- S[4, 2] <- (-1/ns) * (S[2, 2] + S[2, 3])
  S[3, 4] <- S[4, 3] <- (-1/ns) * (S[3, 2] + S[3, 3])
  S[1, 5] <- S[5, 1] <- (-1/ns) * (S[1, 2] + S[1, 4])
  S[2, 5] <- S[5, 2] <- (-1/ns) * (S[2, 2] + S[2, 4])
  S[3, 5] <- S[5, 3] <- (-1/ns) * (S[3, 2] + S[3, 4])
  S[4, 5] <- S[5, 4] <- (-1/ns) * (S[4, 2] + S[4, 4])
  alpha = 1 - cl
  D <- matrix(c(d3_1(SA, SAB, SAG, SE, SB), d3_2(SA, SAB, SAG, 
                                                 SE, SB), d3_3(SA, SAB, SAG, SE, SB), d3_4(SA, SAB, SAG, 
                                                                                           SE, SB), d3_5(SA, SAB, SAG, SE, SB)), nrow = 1)
  varcomp <- c(SA, SAB, SAG, SB, SE)
  names(varcomp) <- c("Subjects", "Subjects-Method", "Subjects-Time", 
                      "Method", "Error")
  est <- ic.ccc(ccc, D, S, alpha)
  res <- list(ccc = est, vc = varcomp, sigma = S, model = model, model_full = model_full)
  class(res) <- "ccc"
  res
}

Lmat<-function(k){
  aux1<-diag(k)*0
  aux1[1,1:k]<-1
  
  for (i in 1:(k-1)) aux1[1+i,1+i]=1
  L=array(0,dim=c(k,k*(k-1)/2))
  cont=0
  for (i in 1:k){
    for (j in 1:(k-1)){
      if (i>j){
        cont=cont+1
        L[,cont]=aux1[,i]-aux1[,j]
      }
    }
  }	
  return(L)
}

varB<-function(model,nobs) model$varFix[1:nobs,1:nobs]

#Computes variability between observers' means
sb<-function(k,b,varB){
  
  L<-Lmat(k)   # Building L matrix
  
  difmed=t(L)%*%b  # Calculating observers sum of squares
  
  #vardifmed=t(L)%*%varB%*%L
  A<-L%*%t(L)
  aux2<-(t(difmed)%*%difmed)-sum(diag(A%*%varB))
  sb<-max(aux2/(k*(k-1)),0)
  return(sb)
}

# Variance of sb
var.sb<-function(k,b,varB){
  L<-Lmat(k)    # Building L matrix
  A<-L%*%t(L)
  xx<-( 2*sum(diag( ( (A%*%varB)**2 ) ) )+ 4*( t(b)%*%A%*%varB%*%A%*%b ) ) /((k*(k-1))**2)
  return(xx)
}

# Prepare secondary functions from the cccrm package which are used in ccclon() 
s_exp<-function(tau)  exp(2*(tau))

d_exp<-Deriv(s_exp,"tau")

icc0<-function(sa,se) sa/(sa+se)
icc1<-function(sa,se,sb) sa/(sa+sb+se)
icc2<-function(sa,sab,se,sb) sa/(sa+sab+sb+se)
icc3<-function(sa,sab,sag,se,sb) (sa+sag)/(sa+sab+sag+se+sb)
icc4<-function(sa,sab,sag,se,sb,sumd) (sumd*(sa+sag))/((sumd*(sa+sab+sag+se))+sb)

d0_1<-Deriv(icc0,"sa")
d0_2<-Deriv(icc0,"se")

d1_1<-Deriv(icc1,"sa")
d1_2<-Deriv(icc1,"se")
d1_3<-Deriv(icc1,"sb")

d2_1<-Deriv(icc2,"sa")
d2_2<-Deriv(icc2,"sab")
d2_3<-Deriv(icc2,"se")
d2_4<-Deriv(icc2,"sb")

d3_1<-Deriv(icc3,"sa")
d3_2<-Deriv(icc3,"sab")
d3_3<-Deriv(icc3,"sag")
d3_4<-Deriv(icc3,"se")
d3_5<-Deriv(icc3,"sb")

d4_1<-Deriv(icc4,"sa")
d4_2<-Deriv(icc4,"sab")
d4_3<-Deriv(icc4,"sag")
d4_4<-Deriv(icc4,"se")
d4_5<-Deriv(icc4,"sb")

ic.ccc <-
  function(ccc,dev,S,alpha){
    
    se.ccc <- as.numeric(sqrt(dev%*%S%*%t(dev)))
    
    z<-0.5*log((1+ccc)/(1-ccc))
    se.z<-c(sqrt(  (se.ccc^2)/(((1+ccc)^2)*((1-ccc)^2))  ) )
    ic.z=z+c(-1,1)*qnorm(1-alpha/2)*se.z
    ic.icc=(exp(2*ic.z)-1)/(exp(2*ic.z)+1)
    result<-c(ccc,ic.icc,se.ccc,z,se.z)
    conf.lab=paste((1-alpha)*100,"%",sep="")
    
    names(result)<-c("CCC",paste("LL CI",conf.lab),paste("UL CI",conf.lab),"SE CCC","Z","SE Z")
    return(result)
  }



#########################
##### Model fitting #####
#########################

### Find optimal lambda for box-cox transformation of total_steps_on_day to improve residuals

# Prepare data
# Total steps
ccc_in <- combined_events %>%
  # Select only relevant variables
  select(c('participant_id', 'device', 'measurement_day', 'total_steps_on_day')) %>%
  # Keep only distinct combinations so that each day is featured only once
  distinct(participant_id, device, measurement_day, total_steps_on_day)

# All walking (all steps above 45 steps/minute)
ccc_in_walking <- combined_events %>%
  # Select only relevant variables
  select(c('participant_id', 'device', 'measurement_day', 
           'cadence_lower_steps', 'cadence_upper_steps')) %>%
  # Keep only distinct combinations so that each day is featured only once
  distinct(participant_id, device, measurement_day, 
           cadence_lower_steps, cadence_upper_steps) %>%
  # Create column with sum of total steps from low and high walking
  mutate(walking_steps = cadence_lower_steps + cadence_upper_steps) %>%
  # Remove low and high walking steps columns
  select(-c('cadence_lower_steps', 'cadence_upper_steps'))

# Slower walking (lower half)
ccc_in_slower <- combined_events %>%
  # Select only relevant variables
  select(c('participant_id', 'device', 'measurement_day', 'cadence_lower_steps')) %>%
  # Keep only distinct combinations so that each day is featured only once
  distinct(participant_id, device, measurement_day, cadence_lower_steps)

# Faster walking (upper half)
ccc_in_faster <- combined_events %>%
  # Select only relevant variables
  select(c('participant_id', 'device', 'measurement_day', 'cadence_upper_steps')) %>%
  # Keep only distinct combinations so that each day is featured only once
  distinct(participant_id, device, measurement_day, cadence_upper_steps)

# Extract Lambda
# Total steps
boxcox1 <- MASS::boxcox(lm(ccc_in$total_steps_on_day ~ 1),
                        plotit = FALSE)
boxcox1$x[which.max(boxcox1$y)]

# All walking
# Constant added to avoid negative values
boxcox2 <- MASS::boxcox(lm(ccc_in_walking$walking_steps + 0.00000001 ~ 1),
                        plotit = FALSE)
boxcox2$x[which.max(boxcox2$y)]

# Slower walking
# Constant added to avoid negative values
boxcox3 <- MASS::boxcox(lm(ccc_in_slower$cadence_lower_steps + 0.00000001 ~ 1),
                        plotit = FALSE)
boxcox3$x[which.max(boxcox3$y)]

# Faster walking
# Constant added to avoid negative values
boxcox4 <- MASS::boxcox(lm(ccc_in_faster$cadence_upper_steps + 0.00000001 ~ 1),
                        plotit = FALSE)
boxcox4$x[which.max(boxcox4$y)]

# Update Slower walking data with box-cox transformation
ccc_in_slower <- combined_events %>%
  # Select only relevant variables
  select(c('participant_id', 'device', 'measurement_day', 'cadence_lower_steps')) %>%
  # Keep only distinct combinations so that each day is featured only once
  distinct(participant_id, device, measurement_day, cadence_lower_steps) %>%
  # Box-cox transformation with lambda = 0.4
  mutate(cadence_lower_steps = (cadence_lower_steps ^ 0.4 - 1) / 0.4)



### Estimate CCC for longitudinal repeated measures 
### using variance components from a linear mixed model

# Total steps
ccc_out <- ccclon_cust(dataset = ccc_in,
                       ry = 'total_steps_on_day', # Outcome
                       rind = 'participant_id', # Subject variable
                       rtime = 'measurement_day', # Time variable
                       rmet = 'device', # Method variable
                       rho = 0, # Within-subject correlation structure (compound symmetry)
                       cl = 0.95) # Confidence level

# All walking (all steps above 45 steps/minute)
ccc_out_walking <- ccclon_cust(dataset = ccc_in_walking,
                               ry = 'walking_steps', # Outcome
                               rind = 'participant_id', # Subject variable
                               rtime = 'measurement_day', # Time variable
                               rmet = 'device', # Method variable
                               rho = 0, # Within-subject correlation structure
                               cl = 0.95) # Confidence level

# Slower walking (lower half)
ccc_out_slower <- ccclon_cust(dataset = ccc_in_slower,
                              ry = 'cadence_lower_steps', # Outcome
                              rind = 'participant_id', # Subject variable
                              rtime = 'measurement_day', # Time variable
                              rmet = 'device', # Method variable
                              rho = 0, # Within-subject correlation structure
                              cl = 0.95) # Confidence level

# Faster walking (upper half)
ccc_out_faster <- ccclon_cust(dataset = ccc_in_faster,
                              ry = 'cadence_upper_steps', # Outcome
                              rind = 'participant_id', # Subject variable
                              rtime = 'measurement_day', # Time variable
                              rmet = 'device', # Method variable
                              rho = 0, # Within-subject correlation structure
                              cl = 0.95) # Confidence level

# Get summaries
summary(ccc_out)
summary(ccc_out_walking)
summary(ccc_out_slower)
summary(ccc_out_faster)



#############################
##### Model diagnostics #####
#############################

### Plot residuals

# Total steps model
plot(ccc_out$model_full)
qqnorm(ccc_out$model_full, ~resid(., type = 'p'), abline = c(0,1))
hist(residuals(ccc_out$model_full), breaks = 20)

# All walking model
plot(ccc_out_walking$model_full)
qqnorm(ccc_out_walking$model_full, abline = c(0,1))
hist(residuals(ccc_out_walking$model_full), breaks = 20)

# Slower walking model
plot(ccc_out_slower$model_full)
qqnorm(ccc_out_slower$model_full, abline = c(0,1))
hist(residuals(ccc_out_slower$model_full), breaks = 20)

# Faster walking model
plot(ccc_out_faster$model_full)
qqnorm(ccc_out_faster$model_full, abline = c(0,1))
hist(residuals(ccc_out_faster$model_full), breaks = 20)



### Plot random effects
# Total steps model
plot(ranef(ccc_out$model_full))

# All walking model
plot(ranef(ccc_out_walking$model_full))

# Slower walking model
plot(ranef(ccc_out_slower$model_full))

# Faster walking model
plot(ranef(ccc_out_faster$model_full))



#######################
##### CCC Results #####
#######################

### Get CCCs
# Total steps
ccc_out_ccc <- as.data.frame(ccc_out$ccc[1:4]) %>%
  rownames_to_column(var = 'ccc_out_ccc$ccc[1:4]') %>%
  rename('Metric' = 'ccc_out_ccc$ccc[1:4]',
         'Value' = 'ccc_out$ccc[1:4]') %>%
  mutate_if(is.numeric, round, 4)

# All walking (all steps above 45 steps/minute)
ccc_out_ccc_walking <- as.data.frame(ccc_out_walking$ccc[1:4]) %>%
  rename('All walking' = 'ccc_out_walking$ccc[1:4]') %>%
  mutate_if(is.numeric, round, 4)

# Slower walking (lower half)
ccc_out_ccc_slower <- as.data.frame(ccc_out_slower$ccc[1:4]) %>%
  rename('Slower walking' = 'ccc_out_slower$ccc[1:4]') %>%
  mutate_if(is.numeric, round, 4)

# Faster walking (upper half)
ccc_out_ccc_faster <- as.data.frame(ccc_out_faster$ccc[1:4]) %>%
  rename('Faster walking' = 'ccc_out_faster$ccc[1:4]') %>%
  mutate_if(is.numeric, round, 4)

# Combine all CCCs in one data frame
ccc_out_ccc_all <- ccc_out_ccc %>%
  bind_cols(ccc_out_ccc_walking) %>%
  bind_cols(ccc_out_ccc_slower) %>%
  bind_cols(ccc_out_ccc_faster) %>%
  rename('All daily steps' = Value) %>%
  select(-c('Metric'))



### Get variance components
# Total steps
ccc_out_vc <- as.data.frame(ccc_out$vc) %>%
  rownames_to_column(var = 'Variance component') %>%
  rename(Variance = 'ccc_out$vc') %>%
  mutate(Percent = Variance / sum(Variance) * 100) %>%
  mutate_if(is.numeric, round, 2) %>%
  mutate(across('Variance', round, 0),
         Variance = scales::comma(Variance))

# All walking (all steps above 45 steps/minute)
ccc_out_vc_walking <- as.data.frame(ccc_out_walking$vc) %>%
  rownames_to_column(var = 'Variance component') %>%
  rename(Variance = 'ccc_out_walking$vc') %>%
  mutate(Percent = Variance / sum(Variance) * 100) %>%
  mutate_if(is.numeric, round, 2) %>%
  select(-1)

# Slower walking (lower half)
ccc_out_vc_slower <- as.data.frame(ccc_out_slower$vc) %>%
  rownames_to_column(var = 'Variance component') %>%
  rename(Variance = 'ccc_out_slower$vc') %>%
  mutate(Percent = Variance / sum(Variance) * 100) %>%
  mutate_if(is.numeric, round, 2) %>%
  mutate(across('Variance', round, 0),
         Variance = scales::comma(Variance)) %>%
  select(-1)

# Faster walking (upper half)
ccc_out_vc_faster <- as.data.frame(ccc_out_faster$vc) %>%
  rownames_to_column(var = 'Variance component') %>%
  rename(Variance = 'ccc_out_faster$vc') %>%
  mutate(Percent = Variance / sum(Variance) * 100) %>%
  mutate_if(is.numeric, round, 2) %>%
  mutate(across('Variance', round, 0),
         Variance = scales::comma(Variance)) %>%
  select(-1)

# Combine variance components in one data frame
ccc_out_vc_all <- ccc_out_vc %>%
  bind_cols(ccc_out_vc_walking) %>%
  bind_cols(ccc_out_vc_slower) %>%
  bind_cols(ccc_out_vc_faster) %>%
  # Update variance components names
  mutate('Variance component' = replace('Variance component', 1:5, 
                                        c('Participant', 
                                          'Participant-System',
                                          'Participant-Day', 
                                          'System',
                                          'Random error')))

# Fix column names
colnames(ccc_out_vc_all) <- c('Variance component', 'Variance', 'Percent', 'Variance', 
                              'Percent', 'Variance', 'Percent', 'Variance', 'Percent')


# Make CCC table
kable(ccc_out_ccc_all, 
      caption = 'CCC and uncertainty statistics for longitudinal repeated measures LMM',
      booktabs = TRUE) %>%
  column_spec(1, width = '3cm', bold = TRUE) %>%
  column_spec(2, width = '3.5cm') %>%
  column_spec(3, width = '3.5cm') %>%
  column_spec(4, width = '3.5cm') %>%
  column_spec(5, width = '3.5cm') %>%
  kable_styling(latex_options = c('striped', 'hold_position'),
                full_width = FALSE)



##############################
##### CCC Visualisations #####
##############################

### Plot proportions of variance

# Prepare data, this time keeping unique column names to create long data frame for ggplot
# Total steps
ccc_out_vc2 <- as.data.frame(ccc_out$vc) %>%
  rownames_to_column(var = 'Variance component') %>%
  rename(Variance = 'ccc_out$vc') %>%
  mutate(Percent = Variance / sum(Variance) * 100) %>%
  mutate(cadence = rep('All daily steps', 5)) %>%
  mutate_if(is.numeric, round, 2) %>%
  # Update variance components names
  mutate('Variance component' = replace('Variance component', 1:5, 
                                        c('Participant', 'Participant-System',
                                          'Participant-Day', 'System',
                                          'Random error')))

# All walking (all steps above 45 steps/minute)
ccc_out_vc_walking2 <- as.data.frame(ccc_out_walking$vc) %>%
  rownames_to_column(var = 'Variance component') %>%
  rename(Variance = 'ccc_out_walking$vc') %>%
  mutate(Percent = Variance / sum(Variance) * 100) %>%
  mutate(cadence = rep('All walking', 5)) %>%
  mutate_if(is.numeric, round, 2) %>%
  # Update variance components names
  mutate('Variance component' = replace('Variance component', 1:5, 
                                        c('Participant', 'Participant-System',
                                          'Participant-Day', 'System',
                                          'Random error')))

# Slower walking (lower half)
ccc_out_vc_slower2 <- as.data.frame(ccc_out_slower$vc) %>%
  rownames_to_column(var = 'Variance component') %>%
  rename(Variance = 'ccc_out_slower$vc') %>%
  mutate(Percent = Variance / sum(Variance) * 100) %>%
  mutate(cadence = rep('Slower walking', 5)) %>%
  mutate_if(is.numeric, round, 2) %>%
  # Update variance components names
  mutate('Variance component' = replace('Variance component', 1:5, 
                                        c('Participant', 'Participant-System',
                                          'Participant-Day', 'System',
                                          'Random error')))

# Faster walking (upper half)
ccc_out_vc_faster2 <- as.data.frame(ccc_out_faster$vc) %>%
  rownames_to_column(var = 'Variance component') %>%
  rename(Variance = 'ccc_out_faster$vc') %>%
  mutate(Percent = Variance / sum(Variance) * 100) %>%
  mutate(cadence = rep('Faster walking', 5)) %>%
  mutate_if(is.numeric, round, 2) %>%
  # Update variance components names
  mutate('Variance component' = replace('Variance component', 1:5, 
                                        c('Participant', 'Participant-System',
                                          'Participant-Day', 'System',
                                          'Random error')))

# Combine data into long data frame for plotting
ccc_out_vc_all_plot <- ccc_out_vc2 %>%
  bind_rows(ccc_out_vc_walking2) %>%
  bind_rows(ccc_out_vc_slower2) %>%
  bind_rows(ccc_out_vc_faster2) %>%
  # Rename for easier plotting reference
  rename(variance_component = 'Variance component',
         percent = Percent,
         variance = Variance) %>%
  mutate_at(vars('variance', 'percent'),
            list(as.numeric)) %>%
  # Define factors
  mutate(cadence = factor(cadence, 
                          levels = c('All daily steps', 
                                     'All walking',
                                     'Slower walking',
                                     'Faster walking'))) %>%
  mutate(variance_component = factor(variance_component, 
                                     levels = c('Participant',
                                                'Participant-System',
                                                'Participant-Day',
                                                'System',
                                                'Random error')))
# Plot
ggplot(data = ccc_out_vc_all_plot, 
       aes(x = cadence,
           y = percent, 
           fill = variance_component)) +
  geom_bar(stat = 'identity', 
           position = position_dodge(width = 0.7), 
           width = 0.6) +
  geom_text(aes(label = round(percent, 0)), 
            position = position_dodge(width = 0.7),
            vjust = -0.75,
            size = 3) +
  scale_x_discrete(labels = c('Total steps',
                              'All walking',
                              'Slower-paced \n walking',
                              'Faster-paced \nwalking'))  +
  scale_y_continuous(limits = c(0, 100),
                     breaks = scales::pretty_breaks(n = 10)) +
  labs(x = '', 
       y = '% of total variance') +
  guides(fill = guide_legend(title = 'Variance component')) +
  theme_bw()



### Scatter plots to illustrate the CCC's accuracy and precision dimensions for 
### each of the four models

### Total steps
# Prepare thigh data
thigh_daily_steps <- combined_events %>%
  filter(device == 'Thigh') %>%
  select(c('participant_id', 'date', 'total_steps_on_day')) %>%
  distinct(participant_id, date, total_steps_on_day)

# Prepare wrist data
wrist_daily_steps <- combined_events %>%
  filter(device == 'Wrist') %>%
  select(c('participant_id', 'date', 'total_steps_on_day')) %>%
  distinct(participant_id, date, total_steps_on_day)

# Combine data
combined_daily_steps <- left_join(thigh_daily_steps, wrist_daily_steps,
                                  by = c('participant_id', 'date')) %>%
  rename(thigh_steps_on_day = total_steps_on_day.x,
         wrist_steps_on_day = total_steps_on_day.y)

# Get Pearson correlation
cor.test(combined_daily_steps$thigh_steps_on_day, 
         combined_daily_steps$wrist_steps_on_day,
         method = 'pearson',
         conf.level = 0.95)

# Plot
rel_plot1 <- ggplot(data = combined_daily_steps, 
                    aes(x = thigh_steps_on_day,
                        y = wrist_steps_on_day)) +
  geom_point(alpha = 0.5,
             size = 0.7) +
  geom_smooth(colour = 'red',
              formula = y ~ x,
              method = 'lm',
              se = FALSE,
              size = 0.5,
              linetype = 'dashed') +
  geom_abline(intercept = 0, 
              slope = 1, 
              linewidth = 0.5,
              colour = 'grey50') +
  scale_x_continuous(breaks = scales::pretty_breaks(n = 4),
                     labels = scales::comma) +
  scale_y_continuous(breaks = scales::pretty_breaks(n = 4),
                     labels = scales::comma) +
  coord_cartesian(xlim = c(0, 30000),
                  ylim = c(0, 30000)) +
  labs(title = '(A) Total steps',
       x = '',
       y = 'Wrist-worn steps') +
  guides(colour = guide_legend(title = 'Type of fit')) +
  annotate(geom = 'text',
           x = 1000,
           y = 28000,
           hjust = 0,
           label = 'PCC: 0.89',
           colour = 'black') +
  annotate(geom = 'text',
           x = 1000,
           y = 25000,
           hjust = 0,
           label = 'CCC: 0.88',
           colour = 'black') +
  theme_bw()



### All walking
# Prepare thigh data
thigh_walking_steps <- combined_events %>%
  filter(device == 'Thigh') %>%
  # Select only relevant variables
  select(c('participant_id', 'device', 'date', 
           'cadence_lower_steps', 'cadence_upper_steps')) %>%
  # Keep only distinct combinations so that each day is featured only once
  distinct(participant_id, device, date, 
           cadence_lower_steps, cadence_upper_steps) %>%
  # Create column with sum of total steps from low and high walking
  mutate(walking_steps = cadence_lower_steps + cadence_upper_steps) %>%
  # Remove low and high walking steps columns
  select(-c('cadence_lower_steps', 'cadence_upper_steps'))

# Prepare wrist data
wrist_walking_steps <- combined_events %>%
  filter(device == 'Wrist') %>%
  # Select only relevant variables
  select(c('participant_id', 'device', 'date', 
           'cadence_lower_steps', 'cadence_upper_steps')) %>%
  # Keep only distinct combinations so that each day is featured only once
  distinct(participant_id, device, date, 
           cadence_lower_steps, cadence_upper_steps) %>%
  # Create column with sum of total steps from low and high walking
  mutate(walking_steps = cadence_lower_steps + cadence_upper_steps) %>%
  # Remove low and high walking steps columns
  select(-c('device', 'cadence_lower_steps', 'cadence_upper_steps'))

# Combine data
combined_walking_steps <- left_join(thigh_walking_steps, wrist_walking_steps,
                                    by = c('participant_id', 'date')) %>%
  rename(thigh_walking_steps = walking_steps.x,
         wrist_walking_steps = walking_steps.y)

# Get Pearson correlation
cor.test(combined_walking_steps$thigh_walking_steps, 
         combined_walking_steps$wrist_walking_steps,
         method = 'pearson',
         conf.level = 0.95)

# Plot
rel_plot2 <- ggplot(data = combined_walking_steps, 
                    aes(x = thigh_walking_steps,
                        y = wrist_walking_steps)) +
  geom_point(alpha = 0.5,
             size = 0.7) +
  geom_smooth(colour = 'red',
              formula = y ~ x,
              method = 'lm',
              se = FALSE,
              size = 0.5,
              linetype = 'dashed') +
  geom_abline(intercept = 0, 
              slope = 1, 
              linewidth = 0.5,
              colour = 'grey50') +
  scale_x_continuous(breaks = scales::pretty_breaks(n = 4),
                     labels = scales::comma) +
  scale_y_continuous(breaks = scales::pretty_breaks(n = 4),
                     labels = scales::comma) +
  coord_cartesian(xlim = c(0, 30000),
                  ylim = c(0, 30000)) +
  labs(title = '(B) All walking',
       x = '',
       y = '') +
  guides(colour = guide_legend(title = 'Type of fit')) +
  annotate(geom = 'text',
           x = 1000,
           y = 28000,
           hjust = 0,
           label = 'PCC: 0.90',
           colour = 'black') +
  annotate(geom = 'text',
           x = 1000,
           y = 25000,
           hjust = 0,
           label = 'CCC: 0.61',
           colour = 'black') +
  theme_bw()



### Slower walking
# Prepare thigh data
thigh_slower_steps <- combined_events %>%
  filter(device == 'Thigh') %>%
  select(c('participant_id', 'date', 'cadence_lower_steps')) %>%
  distinct(participant_id, date, cadence_lower_steps)

# Prepare wrist data
wrist_slower_steps <- combined_events %>%
  filter(device == 'Wrist') %>%
  select(c('participant_id', 'date', 'cadence_lower_steps')) %>%
  distinct(participant_id, date, cadence_lower_steps)

# Combine data
combined_slower_steps <- left_join(thigh_slower_steps, wrist_slower_steps,
                                   by = c('participant_id', 'date')) %>%
  rename(thigh_slower_steps = cadence_lower_steps.x,
         wrist_slower_steps = cadence_lower_steps.y)

# Get Pearson correlation
cor.test(combined_slower_steps$thigh_slower_steps, 
         combined_slower_steps$wrist_slower_steps,
         method = 'pearson',
         conf.level = 0.95)

# Plot
rel_plot3 <- ggplot(data = combined_slower_steps, 
                    aes(x = thigh_slower_steps,
                        y = wrist_slower_steps)) +
  geom_point(alpha = 0.5,
             size = 0.7) +
  geom_smooth(colour = 'red',
              formula = y ~ x,
              method = 'lm',
              se = FALSE,
              size = 0.5,
              linetype = 'dashed') +
  geom_abline(intercept = 0, 
              slope = 1, 
              linewidth = 0.5,
              colour = 'grey50') +
  scale_x_continuous(breaks = scales::pretty_breaks(n = 4),
                     labels = scales::comma) +
  scale_y_continuous(breaks = scales::pretty_breaks(n = 4),
                     labels = scales::comma) +
  coord_cartesian(xlim = c(0, 8250),
                  ylim = c(0, 8250)) +
  labs(title = '(C) Slower-paced walking',
       x = 'Thigh-worn steps',
       y = 'Wrist-worn steps') +
  guides(colour = guide_legend(title = 'Type of fit')) +
  annotate(geom = 'text',
           x = 200,
           y = 7750,
           hjust = 0,
           label = 'PCC: 0.18',
           colour = 'black') +
  annotate(geom = 'text',
           x = 200,
           y = 6950,
           hjust = 0,
           label = 'CCC: 0.14',
           colour = 'black') +
  theme_bw()



#### Faster walking
# Prepare thigh data
thigh_faster_steps <- combined_events %>%
  filter(device == 'Thigh') %>%
  select(c('participant_id', 'date', 'cadence_upper_steps')) %>%
  distinct(participant_id, date, cadence_upper_steps)

# Prepare wrist data
wrist_faster_steps <- combined_events %>%
  filter(device == 'Wrist') %>%
  select(c('participant_id', 'date', 'cadence_upper_steps')) %>%
  distinct(participant_id, date, cadence_upper_steps)

# Combine data
combined_faster_steps <- left_join(thigh_faster_steps, wrist_faster_steps,
                                   by = c('participant_id', 'date')) %>%
  rename(thigh_faster_steps = cadence_upper_steps.x,
         wrist_faster_steps = cadence_upper_steps.y)

# Get Pearson correlation
cor.test(combined_faster_steps$thigh_faster_steps, 
         combined_faster_steps$wrist_faster_steps,
         method = 'pearson',
         conf.level = 0.95)

# Plot
rel_plot4 <- ggplot(data = combined_faster_steps, 
                    aes(x = thigh_faster_steps,
                        y = wrist_faster_steps)) +
  geom_point(alpha = 0.5,
             size = 0.7) +
  geom_smooth(colour = 'red',
              formula = y ~ x,
              method = 'lm',
              se = FALSE,
              size = 0.5,
              linetype = 'dashed') +
  geom_abline(intercept = 0, 
              slope = 1, 
              linewidth = 0.5,
              colour = 'grey50') +
  scale_x_continuous(breaks = scales::pretty_breaks(n = 4),
                     labels = scales::comma) +
  scale_y_continuous(breaks = scales::pretty_breaks(n = 4),
                     labels = scales::comma) +
  coord_cartesian(xlim = c(0, 30000),
                  ylim = c(0, 30000)) +
  labs(title = '(D) Faster-paced walking',
       x = 'Thigh-worn steps',
       y = '') +
  guides(colour = guide_legend(title = 'Type of fit')) +
  annotate(geom = 'text',
           x = 1000,
           y = 28000,
           hjust = 0,
           label = 'PCC: 0.83',
           colour = 'black') +
  annotate(geom = 'text',
           x = 1000,
           y = 25000,
           hjust = 0,
           label = 'CCC: 0.55',
           colour = 'black') +
  theme_bw()


# Combine plots
rel_plot1 + rel_plot2 + rel_plot3 + rel_plot4 +
  plot_layout(ncol = 2)
